#!/usr/bin/perl -w
#
# Get TV listings for http://www.bleb.org/tv/, and generates Perl code
# containing the data.
#
# (c) Andrew Flegg 2000. Thanks to Graham Chapman for further work.

use strict;
use lib "/home/jaffa/bin/tv";
use Telly;
use vars qw($DEBUG $WEEK);

my $cli = ":" . join(":", @ARGV) . ":";
$DEBUG = ($cli =~ m/:-debug:/i);
$WEEK  = ($cli =~ m/:-week:/i);

my $LOOKING_FOR_FIRST = 0;
my $LOOKING_FOR_TIME  = 1;
my $LOOKING_FOR_TITLE = 2;
my $READING_DESC      = 3;

# -- Process each channel in a separate thread...
#
foreach my $i (keys(%Telly::CHANNELS)) {
    my $j     = $Telly::CHANNELS{$i};
    my $child = 0; # fork();
#    $child = fork();

    if ($child == 0) {
        my $dir = $DATA;
        &doChannel("$BASE/$i", "$dir/+0", $j);
        if ($WEEK) {
	    for(my $day=1;$day<=6;$day++) {
                my $lcl = "$BASE/$i";
                eval('$lcl =~ s/day1/day'.($day+1).'/;');
		print "$lcl => +$day/$j\n" if $DEBUG;
                &doChannel($lcl, "$dir/+$day", $j);
            }
        }
    }
}
exit;


sub doChannel {
    my ($i, $dir, $j) = @_;

    my $cmd = "$LYNX $i.html";
    open(IN, "$cmd |") or die "getchannels: failed with $cmd\n";
    open(OUT, ">$dir/$j.pm");
    if (! \*OUT) {
        close(IN);
        die "getchannels: couldn't write $j.dat\n";
    }

    print "Processing $i => $dir/$j...\n" if $DEBUG;
    my $date = scalar(localtime);
    print OUT <<EOH;
# Today's Telly information for $j
#
# Automatically generated at $date
# (c) Andrew Flegg 2000. All rights reserved.

my \@channel;
EOH

	# -- Process the input
        #
	my $status = $LOOKING_FOR_TIME;
        my $time   = "";
	my $title  = "";
	my $desc   = "";
	my $count  = 0;
	while (<IN>) {
	    if ($status == $LOOKING_FOR_TIME) {
		if (m!<b>(\d{1,2}\.\d{2}[ap]m)?</b>!i) {
		    # -- Found a new program
		    #
		    if ($1) {
			$time = time24($1);
		    } else {
			$time = "";
		    }
		    $title  = "";
		    $desc   = "";
		    $status = $LOOKING_FOR_TITLE;
                    print "$time\t: " if $DEBUG;
		}
	    
	    } elsif ($status == $LOOKING_FOR_TITLE) {
		if (m!\s*<b>([^<>]+)</b>\s*$!i) {
		    # -- Got the title
		    #
		    $title  = $1;
		    $status = $READING_DESC;
                    print "$title\n" if $DEBUG;
		}

	    } elsif ($status == $READING_DESC) {
		if (m!^\s*(<[ib]>)?([^<>]*)(</[ib]>)?\s*$!i) {
		    # -- A new valid line of description
		    #
		    my $this = lc($1).$2.lc($3);
		    if ($this ne "") {
			$this =~ s/&/&amp;/g;
			$desc .= $this."\n";
		    }
		} elsif (m!</small></td></tr>!i) {
		    # -- End of record
		    #
		    #$desc = quotemeta($desc);
		    $title =~ s/'/\\'/g;
		    $desc  =~ s/'/\\'/g;
		    print OUT "\$channel[$count]\t= ['$time',\n".
			      "         \t  '$title',\n".
			      "         \t  '$desc'];\n";
		    $status = $LOOKING_FOR_TIME;
		    $count++;
		}
	    }
	}

    print OUT "\n\n\$Telly::LISTINGS{'$j'} = \\\@channel;\n1;\n";
    close(OUT);
    close(IN);
}



