#!/usr/bin/perl -w
#
# Get TV listings for http://www.bleb.org/tv/, and generates Perl code
# containing the data.
#
# (c) Andrew Flegg 2000. Thanks to Graham Chapman for further work.

use strict;
use lib "/home/jaffa/bin/tv";
use Telly;
use LWP::Simple;
use vars qw($DEBUG $WEEK $SHIFT);

my $cli = ":" . join(":", @ARGV) . ":";
$DEBUG = ($cli =~ m/:-debug:/i);
$WEEK  = ($cli =~ m/:-week:/i);
$SHIFT = ($cli =~ m/:-shift:/i);

my %rev  = ();
while (my ($k, $v) = each(%Telly::CHANNELS)) { $rev{$v} = $k; }
my @todo = grep(!/^-/, @ARGV);
@todo    = keys(%Telly::CHANNELS) unless @todo;

# -- Shift existing data back down as a backup...
#
if ($SHIFT) {
    my $now = time;
    system "rm -rf \"$DATA/-1\"";
    for (my $day=0; $day<=6; $day++) {
        my $target = (--$day < 0 ? '' : '+').$day++;
        print "Renaming $DATA/+$day to $target...\n" if $DEBUG;
        rename "$DATA/+$day", "$DATA/$target";
        utime $now, $now, <$DATA/$target/*.pm>;
    }
    mkdir "$DATA/+6", 0755;
}

# -- Process each channel in a separate thread...
#
foreach my $i (@todo) {
    my $j     = $Telly::CHANNELS{$i};
    ($i, $j)  = ($rev{$i}, $i) unless $j;
    my $child = 0; # fork();

    if ($child == 0) {
        my $dir = $DATA;
        &doChannel("$BASE$i", "$dir/+0", $j);
        if ($WEEK) {
	    for(my $day=1;$day<=6;$day++) {
                my $lcl = "$BASE$i";
                eval('$lcl =~ s/day1/day'.($day+1).'/;');
		print "$lcl => +$day/$j\n" if $DEBUG;
		sleep 2;
                &doChannel($lcl, "$dir/+$day", $j);
            }
        }
    }
    sleep 5;
}
exit;


sub doChannel {
    my ($i, $dir, $j) = @_;

    print "Fetching $i...\n" if $DEBUG;
    my $data = get($i);
    print "Unable to fetch $i\n", return unless $data;
    unless(open(OUT, ">$dir/$j-new.pm")) {
        print "getchannels: couldn't write $j-new.pm - '$!'\n";
	return;
    }

    $data =~ s/\r//g;
    my @in   = split(/\n/, $data);
    print "Processing $i => $dir/$j...\n" if $DEBUG;
    my $date = scalar(localtime);
    print OUT <<EOH;
# Today's Telly information for $j
#
# Automatically generated at $date
# (c) Andrew Flegg 2000. All rights reserved.

my \@channel;
EOH

	# -- Process the input
        #
        my $time   = "";
	my $title  = "";
	my $desc   = "";
        my $url    = "";
	my $count  = 0;
	foreach (@in) {
	    next unless m!<b>(\d{1,2}\D\d{2}[ap]m)</b>!i;

	    $time = time24($1);
	    ($title, $desc) = m!"top"><small>.*?<b>(.*?)</b> (.*)</small></td>!i;

            if (m!<a[^>]+href="[^\"]*tv_det.html\?t=(\d+)"[^>]*>!) {
                  # -- Found the "more details" URL
                  #
                  print "$1 " if $DEBUG;
                  $url = "info.cgi?id=$1";
		  $desc .= "<small><b><a href=\"$url\">[&gt;&gt;]</a></b></small>";
             }

            print "$time : $title... " if $DEBUG;
	    $title =~ s/'/\\'/g;
	    $desc  =~ s/'/\\'/g;

	    print OUT "\$channel[$count]\t= ['$time',\n".
		      "         \t  '$title',\n".
		      "         \t  '$desc'];\n";
	    $count++;
            print "done.\n" if $DEBUG;
	}

    print OUT "\n\n\$Telly::LISTINGS{'$j'} = \\\@channel;\n1;\n";
    close(OUT);
    if ($count) {
        unlink "$dir/$j.pm";
        rename "$dir/$j-new.pm", "$dir/$j.pm";
    } else {
	my $reason = $data;
        $reason = $1 if $data =~ m#</form>\s*<!--(.*)-->\s*</td>#s;

        unlink "$dir/$j-new.pm";
        print "+++ Warning: $dir/$j has no programmes...\n$reason";
    }
}



