package Telly;

use strict;

BEGIN {
    use Exporter   ();
    use vars       qw($VERSION @ISA @EXPORT);

    # set the version for version checking
    $VERSION     = 1.03;
    @ISA         = qw(Exporter);
    @EXPORT      = qw($BASE $LYNX $DATA %CHANNELS %LISTINGS
		      &channelList
		      &niceChannelName
		      &niceTime
		      &time24
		      &registerErrorHandler
		      &progGetTime
		      &progGetTitle
		      &progGetDescription
		      &onAt
		     );
}
use vars      @EXPORT;


# -- Constants --------------------------------------------------
#
#$BASE     = 'http://www.ananova.com/tv_listings/tv_mainlisting.html?day=day1&start=Start&end=End&c=';
$BASE     = 'http://www.ananova.com/tv_listings/tv_channel.html?day=day1&c=';
$LYNX     = '/usr/bin/lwp-request';
$DATA     = '/home/jaffa/bleb/tv/data';
#$DATA     = '/dcs/97/csube/tmp/tv';
%CHANNELS = ( "bbc1_s_9" => "bbc1",
              "bbc2_s_9" => "bbc2",
              "49" => "itv",
              "ch4" => "ch4",
              "ch5" => "ch5",
	      "bbcchoice" => "bbc_choice",
	      "742" => "bbc4",
              "itv2" => "itv2",
              "sky_channel" => "sky1",
	      "e4" => "e4",
              "ukgold" => "uk_gold",
	      "110" => "granada_plus",
	      "119" => "men_and_motors",
              "102" => "cartoon_network",
              "275" => "british_eurosport",
              "169" => "discovery",
              "paramount" => "paramount",
              "188" => "nickelodeon",
              "bsb_sports" => "sky_sports1",
              "sky_sports2" => "sky_sports2",
	      "220" => "film_four",
              "sky_sports3" => "sky_sports3",
              "sky_movies" => "sky_moviemax",
              "bsb_movie" => "sky_premier" );
%LISTINGS = ();
# ---------------------------------------------------------------

# -- Internal variables -----------------------------------------
my @TERRESTRIAL  = ( "bbc1", "bbc2", "itv", "ch4", "ch5" );
my $errorHandler = \&error;
# ---------------------------------------------------------------


# -- Register a new error handler
#
sub registerErrorHandler {
    ($errorHandler) = @_;
}


# -- The default error handler
#
sub error {
    die "Telly.pm: Error - $_[0]\n";
}


# -- Convert a time (eg. "6.00am") into a 24 hour time
#
sub time24 {
    my ($str) = lc($_[0]);
    $str =~ m/^(\d+)\D?(\d+)([ap]m)?/i;

    my $time = $1;
    if (lc($3) eq "pm") {
	$time += 12 unless $time == 12;
    } else {
	$time = 0 if ($time == 12) and (lc($3) eq "am");
    }
    $time .= $2;
    return sprintf("%4.4d", $time);
}


# -- Convert the output of 'time24' into printable form
#
sub niceTime {
    my ($time) = @_;
    if ($time =~ m/^(\d{2})(\d{2})$/) {
	return "$1:$2";
    }

    return $time;
}


#-- Get a list of channels in "nice" order,
#   ie. BBC 1, BBC 2, ITV, Ch 4, Ch5 and then the rest in
#   alphabetical order
#
sub channelList {
    my @list     = sort(values(%CHANNELS));
    my @out      = @TERRESTRIAL;
    my %contains = ();
    for (@out) { $contains{$_} = 1; }

    foreach my $i (sort(@list)) {
        if (! $contains{$i}) {
            $out[$#out+1] = $i;
            $contains{$i} = 1;
        }
    }
    return @out;
}


#-- Get a nice name for a channel ----------------------------------
#
sub niceChannelName {
    my ($nice) = @_;
    $nice =~ s/(\w{2,})(\d)/$1 $2/g;
    $nice =~ s/_/ /g;
    $nice =~ s/bbc/BBC/g;
    $nice =~ s/itv/ITV/g;
    $nice =~ s/ch /Channel /g;
    $nice =~ s/uk /UK /g;
    $nice =~ s/ (.)/" ".ucfirst($1)/eg;
    $nice = ucfirst($nice);
 
    return $nice;
}


#-- Get a programmme reference's time ---------------------------------
#
sub progGetTime {
    my ($prog) = @_;
    my $val    = @{ $prog }[0];

    return ($val ? $val : 0);
}


# -- Get a programme reference's title -------------------------------
#
sub progGetTitle {
    my ($prog) = @_;

    return @{ $prog }[1];
}


# -- Get a programme reference's description -------------------------
#
sub progGetDescription {
    my ($prog) = @_;

    return @{ $prog }[2];
}


# -- Get an array of programs on before, at and after a time -------
#    on a particular channel
#
sub onAt {
    my ($channel, $time) = @_;

    my ($previous, $at, $after) = (-1, -1, -1);
    my @progs = @{ $LISTINGS{$channel} };
    &$errorHandler("Invalid channel '$channel'") unless @progs;

    for(my $i = 0; $i < $#progs; $i++) {
	$after = $i if (($after == -1) && ($time < progGetTime($progs[$i])));
    }

    ($previous, $at) = ($after - 2, $after - 1);
    $previous = -1 if ($previous < 0);
    $at       = -1 if ($at < 0);

    $previous = getAtRef($channel, $previous);
    $at       = getAtRef($channel, $at);
    $after    = getAtRef($channel, $after);
    return ($previous, $at, $after);
}


# -- Used by onAt to turn array references into programme refs -----
#
sub getAtRef {
    my ($channel, $idx) = @_;

    my @progs = @{ $LISTINGS{$channel} };
    my $ref   = undef;

    print "Starting at $idx...\n";
    while (($idx >= 0) && ($progs[$idx][0] eq "")) {
	$idx--;
	print "Decrementing to $idx...\n";
    }

    $ref = $progs[$idx] if ($idx >= 0);
    return $ref;
}


END {}
# 1; # -- Keep Perl happy ------------------------------------------
